
#ifndef PRRAM_H
#define PRRAM_H

#include "def.h"
#include "xref.h"

void rule_init     ( void                        );
void print_title   ( /*const*/ struct xram *xram );
void print_xref    (   const   struct xram *xram );
void print_ram_foot(   const   struct xram *xram );
void print_ram     ( void                        );

#endif

