
#ifndef CONV_H
#define CONV_H

#include "def.h"

/* Notice that the `char *' functions mutate their referenced
 * arguments.  */
inline char uc1   ( char  c );
inline char lc1   ( char  c );
inline char unlat1( char  c );
inline char undot1( char  c );
char       *uc    ( char *s );
char       *lc    ( char *s );
char       *unlat ( char *s );
char       *undot ( char *s, const char c );

#endif

