
#ifndef SEL_H
#define SEL_H

/* Watch the next line.  It defines large enough a hash for woody and
 * sarge, but Debian might eventually outgrow it.  One hash slot is
 * required per package name to the -s option.  (Hopefully a future
 * version of <search.h> will soften this hard-coded barrier.)  */
#define HASH_SIZE           0x8000
#define SEL_SIZE_LINE        0x200
#define ENFORCE_STR_DPKG_INSTALL 1
#define STR_DPKG_INSTALL   "install"
#define SEL_IO_ERROR_MSG   "improper selection"

#include "def.h"
#include <search.h>

extern struct hsearch_data sel;

int set_selections( void );

#endif

