
#ifndef GENERAL_H
#define GENERAL_H

/* The header `def.h' defines _GNU_SOURCE. */

#include "def.h"
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <error.h>

#endif

