
#ifndef ARGP_H
#define ARGP_H

#include "def.h"

enum type_name_cl { NAME_MAINT = 1, NAME_DEB };

struct ram_cl  {
  struct ram_cl  *next          ;
  char            ram[N_DIG+1]  ;
};
struct name_cl {
  struct name_cl *next          ;
  char           *name          ;
};
struct options  {
  struct ram_cl  *ram_cl_first  ;
  struct name_cl *maint_cl_first;
  struct name_cl *deb_cl_first  ;
  int             wide          ;
  int             no_latin1     ;
  int             utf8          ;
  int             latin1        ;
  int             expand_xref   ;
  int             plan          ;
  int             recursive     ;
  int             recursive_up  ;
  int             no_deb        ;
  int             no_subram     ;
  int             no_title      ;
  int             no_xref       ;
  int             no_maint      ;
  int             no_pri        ;
  int             no_desc       ;
  int             no_count      ;
  int             selections    ;
  int             color         ;
  int             pri_one_color ;
  int             ascii_dots    ;
  char           *data_file_gz  ;
  char           *data_file     ;
};

extern struct options opt       ;
extern char           zero_ram[];
/* The parse_cl() initializes zero_ram[]; thus do not use zero_ram
 * before calling parse_cl().  */

void parse_cl( int argc, char **argv );

#endif

