
/* main.c */

#include "main.h"
#include "argp.h"
#include "sel.h"
#include "utf8.h"
#include "xref.h"
#include "plan.h"
#include "prram.h"
#include "find.h"
#include "gen.h"

int main( int argc, char **argv ) {

  parse_cl( argc, argv );
  locale();
  if ( opt.selections ) set_selections();
  xref_pass();

  if      ( opt.plan                               ) print_plan();
  else if ( opt.maint_cl_first || opt.deb_cl_first ) print_find();
  else                                               print_ram ();

  return 0;

}

