package DebSrc;
use warnings;
use strict;
use integer;
use DebSrcBin;
our @ISA = ( 'DebSrcBin' );

# List the build-dependency fields.
our @dkey_subparsable_bdep = qw(
  Build-Depends
  Build-Depends-Indep
  Build-Conflicts
  Build-Conflicts-Indep
);

# Instantiate and initialize a new DebSrc object.
sub new {
  my $class = shift;
  my $src1  = DebSrcBin->new( @_ );
  bless $src1, $class if defined $src1;
  return $src1;
}

1;

